/*
Copyright (C) 2004 Douglas Thain
This work is made available under a BSD-style license.
See the file COPYING for details.
*/

#include "copy_stream.h"
#include "full_io.h"

#include <stdio.h>
#include <stdlib.h>

#define COPY_BUFFER_SIZE 32768

int copy_stream_to_stream( FILE *input, FILE *output )
{
	char buffer[COPY_BUFFER_SIZE];
	int actual_read=0, actual_write=0;
	int total=0;

	while(1) {
		actual_read = full_fread(input,buffer,COPY_BUFFER_SIZE);
		if(actual_read<=0) break;

		actual_write = full_fwrite(output,buffer,actual_read);
		if(actual_write!=actual_read) {
			total = -1;
			break;
		}

		total+=actual_write;
	}

	if( ( (actual_read<0) || (actual_write<0) ) && total==0 ) {
		return -1;
	} else {
		return total;
	}
}

int copy_stream_to_buffer( FILE *input, char **buffer )
{
	int buffer_size = 8192;
	int total=0;
	int actual;
	char *newbuffer;

	*buffer = malloc(buffer_size);
	if(!*buffer) return -1;

	while(1) {
		actual = full_fread(input,&(*buffer)[total],buffer_size-total);
		if(actual<=0) break;

		total += actual;

		if( (buffer_size-total)<1 ) {
			buffer_size *= 2;
			newbuffer = realloc(*buffer,buffer_size);
			if(!newbuffer) {
				free(*buffer);
				return -1;
			}
			*buffer = newbuffer;
		}
	}

	(*buffer)[total] = 0;

	return total;
}

int copy_stream_to_fd( FILE *input, int fd )
{
	char buffer[COPY_BUFFER_SIZE];
	int actual_read=0, actual_write=0;
	int total=0;

	while(1) {
		actual_read = full_fread(input,buffer,COPY_BUFFER_SIZE);
		if(actual_read<=0) break;

		actual_write = full_write(fd,buffer,actual_read);
		if(actual_write!=actual_read) {
			total = -1;
			break;
		}

		total+=actual_write;
	}

	if( ( (actual_read<0) || (actual_write<0) ) && total==0 ) {
		return -1;
	} else {
		return total;
	}
}

int copy_fd_to_stream( int fd, FILE *output )
{
	char buffer[COPY_BUFFER_SIZE];
	int actual_read=0, actual_write=0;
	int total=0;

	while(1) {
		actual_read = full_read(fd,buffer,COPY_BUFFER_SIZE);
		if(actual_read<=0) break;

		actual_write = full_fwrite(output,buffer,actual_read);
		if(actual_write!=actual_read) {
			total = -1;
			break;
		}

		total+=actual_write;
	}

	if( ( (actual_read<0) || (actual_write<0) ) && total==0 ) {
		return -1;
	} else {
		return total;
	}
}
